/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : I2C_Master.c

 Hardware      : ADuC706x

 Description   : This I2C Master example acts as interface between the PC based application I2CSerial.exe
 				and the Slave EPROM. 
				This application accepts commands via the serial port from the PC, converts these to I2C
				and passes them to the Slave/EPROM device.
				 Using the command window based executable, I2CSerial.exe, data bytes
				are sent serially to the Master via UART. Depending on whether	the application sent
				a write or read command, the Master sends the appropriate command to the Slave via 
				I2C channel. If a slave read is requested, the Master will request the data
				from the I2Cserial.exe specified address and return the data to the PC.

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define I2C_RD_BIT 0x1

#include<aduc7060.h>
// Function Definitions

void delay(int);


unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;  // Array index variable for szTxData[]
unsigned char szRxData[6];	  // Array for reading Data from Slave
unsigned char ucRxCount = 0;  // Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin reading from Slave - controlled by XIRQ0

unsigned char szStoreData[] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55};	// Array to send to Slave
unsigned char ucStoreCount = 0;  // Array index variable for szStoreData[]
int main(void)
{
    POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;

	POWKEY3 = 0x76;
	POWCON0 = 0x124;	   // Set the Peripheral clocks to max speed of 10.24Mhz
	POWKEY4 = 0xB1;

	IRQCONE = BIT1;			// Rising edge trigger on XIRQ0
	// Configure P0.1 and P0.3 for I2C mode. Configure P1.0 and P1.1 for UART mode. 
	GP0CON0 = BIT4 + BIT12; // Select SPI/I2C alternative function for P0.1 & P0.3
	GP0KEY1 = 0x7;		   // Write to GP0KEY1
	GP0CON1	= BIT1;		   // Select I2C functionality for P0.1 & P0.3
	GP0KEY2 = 0x13;		   // Write to GP0KEY2

	// Enable I2C Slave mode, Slave ID and interrupt sources
	I2CID0  = 0xA0;			// Configure Slave's I2C network address
	I2CFSTA = BIT8;			// Flush Slave Tx FIFO
	I2CFSTA &= ~BIT8;
	I2CSCON = BIT0 + BIT4   // Enable I2C Slave + Enable Rx interrupt +
			+ BIT9 + BIT10  // Enable Rx interrupt + Enable Tx interrupt +
			+ BIT10;		// Enable "Stop-after-Start-Detected" IRQ.
	// Enable I2C Master mode, baud rate and interrupt sources
	I2CMCON = BIT0 + BIT4  // Enable I2C Master + Enable Rx interrupt
			+ BIT5 + BIT8; // Enable Tx interrupt + Enable transmission complete interrupt
	I2CDIV  = 0x3131;	   // Select 100.3kHz clock rate
	IRQEN = BIT15 + BIT16 + BIT13; // Enable I2C Master, Slave and XIRQ0 interrupts

	// Begin Master Transmit sequence
	ucTxCount = 0;
	I2CFSTA = BIT9;			// Flush Master Tx FIFO
	I2CFSTA &= ~BIT9;
	I2CMTX = szTxData[ucTxCount++];
	I2CADR0 = 0xA0; 
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
	while (1)
	{
	 	if (ucWaitingForXIRQ0 == 0)	// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			ucRxCount = 0;
			I2CMCNT0 = 5; // Read back 6 bytes
			I2CADR0 =  0xA0 + I2C_RD_BIT; // Begin Read sequence from Slave
		}
	}
}

void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CMSTATUS = 0;
	unsigned int I2CSSTATUS = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
	if ((IRQSTATUS & BIT13) == BIT13)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;			// Enable Read request from Slave.
		IRQCLRE = BIT13;
		IRQCLR = BIT13;
	}

	if ((IRQSTATUS & BIT15) == BIT15)	//If I2C Master interrupt source
	{
	   I2CMSTATUS = I2CMSTA;
	   if ((I2CMSTATUS & BIT2) == BIT2) // If I2C Master Tx IRQ
	   {
			if (ucTxCount < 6)		   // Have max 6 bytes been sent?
				I2CMTX = szTxData[ucTxCount++];	 // Load Tx buffer
	   }
	   if ((I2CMSTATUS & BIT3) == BIT3) // If I2C Master Rx IRQ
	   {
			if (ucRxCount < 6)		   // Have max 6 bytes been received?
			{
				szRxData[ucRxCount] = I2CMRX;  // Read Rx buffer
				ucRxCount++;
			}
	   }
	}
	if ((IRQSTATUS & BIT16) == BIT16)	//If I2C Slave interrupt source
	{
	   I2CSSTATUS = I2CSSTA;
	   if ((I2CSSTATUS & BIT2) == BIT2) // If I2C Slave Tx IRQ
	   {
			if (ucStoreCount < 6)		   // Have max 6 bytes been sent?
				I2CSTX = szStoreData[ucStoreCount++];	 // Load Tx buffer
	   }
	   if ((I2CSSTATUS & BIT3) == BIT3) // If I2C Master Rx IRQ
	   {
			if (ucStoreCount < 6)		   // Have max 6 bytes been received?
			{
				szStoreData[ucStoreCount] = I2CSRX;  // Read Rx buffer
				ucStoreCount++;
			}
	   }
	   if ((I2CSSTATUS & BIT10) == BIT10) // If I2C Stop after Start condition detected IRQ
	   {
			ucStoreCount = 0;	// Reset counter Array pointer.
			I2CFSTA = BIT8;			// Flush Slave Tx FIFO
			I2CFSTA &= ~BIT8;
	   }
	   if ((I2CSSTATUS & BIT13) == BIT13) // If I2C Repeated Start detected IRQ
	   {
			ucStoreCount = 0;	// Reset counter Array pointer.
			I2CFSTA = BIT8;			// Flush Slave Tx FIFO
			I2CFSTA &= ~BIT8;
	   }

	}
}


